"""Provides weather data based on zipcode."""

from os import path
import requests
import json
from typing import Union


class WeatherWise:
    """Obtain local weather information based on zipcode.

    Args:
        zipcode (str): The location's zipcode for weather information.
    """

    def __init__(self, zipcode: str) -> None:
        """Instantiate a Weather Class instance."""
        self.zipcode = zipcode
        self.data = self._load_json_data()
        self.latitude, self.longitude = self._get_latitude_longitude()
        self.forecast_url = self._get_weather_forecast_url()
        self.weather_details = self._get_weather_data()

    def _load_json_data(self) -> dict:
        """Load the JSON data set of zipcodes, latitudes and longitudes.

        Returns:
            dict: Data set of zipcodes, latitudes, and longitudes.
        """
        data_directory = path.join(path.dirname(__file__), "data")
        with open(path.join(data_directory, "2022_gazetteer_zcta_national.json"), "r") as json_file:
            gazetteer_data = json.load(json_file)

            return gazetteer_data

    def _get_latitude_longitude(self) -> tuple:
        """Get latitude and longitude based on the zipcode provided.

        Returns:
            tuple: Latitude and longitude coordinates.
        """
        for dictionary in self.data:
            if self.zipcode == (dictionary["zipcode"]):
                latitude = dictionary["latitude"]
                longitude = dictionary["longitude"]

                return latitude, longitude

        raise ValueError(f"Invalid Zip Code: {self.zipcode}")

    def _get_weather_forecast_url(self) -> str:
        """Get National Weather Forecast Office forecast url.

        Returns:
            str: URL of the local weather office obtained by the latitude and longitude coordinates.
        """
        url = f"https://api.weather.gov/points/{self.latitude},{self.longitude}"

        weather_station_response = requests.get(url, timeout=10)
        weather_station_data = weather_station_response.json()
        forecast_url = weather_station_data["properties"]["forecastHourly"]

        return forecast_url

    def _get_weather_data(self) -> dict:
        """Get current weather forecast data.

        Returns:
            dict: Weather related items.
        """
        weather_data_response = requests.get(self.forecast_url, timeout=10)
        weather_data = weather_data_response.json()
        current_weather = weather_data["properties"]["periods"][0]

        weather_information = {
            "short_forecast": current_weather["shortForecast"],
            "detailed_forecast": current_weather["detailedForecast"],
            "temperature": current_weather["temperature"],
            "temperature_unit": current_weather["temperatureUnit"],
            "probability_of_precipitation": current_weather["probabilityOfPrecipitation"]["value"],
            "wind_speed": current_weather["windSpeed"],
            "wind_direction": current_weather["windDirection"],
        }

        return weather_information

    def get_short_forecast(self) -> str:
        """Get the current short weather forecast.

        Returns:
            str: Current short weather forecast.
        """
        short_forecast = self.weather_details["short_forecast"]

        return short_forecast

    def get_detailed_forecast(self) -> str:
        """Get the current detailed weather forecast.

        Returns:
            str: Current detailed weather forecast.
        """
        detailed_forecast = self.weather_details["detailed_forecast"]

        return detailed_forecast

    def get_temperature_in_fahrenheit(self, temperature_unit: bool = False) -> Union[int, str]:
        """Get the current temperature in fahrenheit.

        Args:
            temperature_unit (bool): Indicate whether to include the temperature unit.

        Returns:
            int: Union[int, str]: Current temperature in Fahrenheit as an integer or a string with the unit.
        """
        temperature_in_fahrenheit = self.weather_details["temperature"]
        if temperature_unit:
            return f"{temperature_in_fahrenheit}F"

        return temperature_in_fahrenheit

    def get_temperature_in_celsius(self, temperature_unit: bool = False) -> Union[float, str]:
        """Get the current temperature in celsius.

        Args:
            temperature_unit (bool): Indicate whether to include the temperature unit.

        Returns:
            Union[float, str]: Current temperature in Celsius as a float or a string with the unit.
        """
        temperature_in_celsius = (self.weather_details["temperature"] - 32) / 1.8
        if temperature_unit:
            return f"{temperature_in_celsius}C"

        return f"{temperature_in_celsius:.2f}"

    def get_probability_of_precipitation(self) -> str:
        """Get the current probability of precipitation.

        Returns:
            str: Current probability of precipitation.
        """
        probability_of_precipitation = self.weather_details["probability_of_precipitation"]

        if probability_of_precipitation is None:
            return "0"

        return f"{probability_of_precipitation}"

    def get_wind_speed(self) -> str:
        """Get the current wind speed.

        Returns:
            str: Current wind speed.
        """
        wind_speed = self.weather_details["wind_speed"]

        return wind_speed

    def get_wind_direction(self) -> str:
        """Get the current wind direction.

        Returns:
            str: Current wind direction.
        """
        wind_direction = self.weather_details["wind_direction"]

        return wind_direction

    def get_wind(self) -> str:
        """Get the current wind speed and wind direction.

        Returns:
            str: Current wind speed and wind direction.
        """
        wind_speed = self.weather_details["wind_speed"]
        wind_direction = self.weather_details["wind_direction"]

        return f"{wind_speed} {wind_direction}"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['weather_wise']

package_data = \
{'': ['*'], 'weather_wise': ['data/*']}

install_requires = \
['requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'weather-wise',
    'version': '0.1.5',
    'description': 'Obtain local weather information based on zipcode.',
    'long_description': '# Weather Wise\n\n## Overview\n\nGet the current weather forecast and other weather related information from a zipcode.  This module uses the U.S. Census 2022 Gazetteer (Zip Code Tabulation Areas).  The original file has been modified to JSON.\n\n## Usage\n\n```python\nimport json\nfrom weather_wise.weather_wise import WeatherWise\n\nweather = WeatherWise("32904")\n\n# Internal weather methods.\nprint(weather._load_json_data())\nprint(weather._get_latitude_longitude())\nprint(weather._get_weather_forecast_url())\nprint(json.dumps(weather._get_weather_data(), indent=2))\n\n# Get the short forecast.\nshort_forecast = weather.get_short_forecast()\nprint(short_forecast)\n\n# Get the temperature in fahrenheit.\ntemperature_in_fahrenheit = weather.get_temperature_in_fahrenheit(temperature_unit=True)\nprint(temperature_in_fahrenheit)\n\n# Get the temperature in celsius.\ntemperature_in_celsius = weather.get_temperature_in_celsius()\nprint(temperature_in_celsius)\n\n# Get the chance of rain.\nchance_of_rain = weather.get_probability_of_precipitation()\nprint(chance_of_rain)\n\n# Get the wind speed.\nwind_speed = weather.get_wind_speed()\nprint(wind_speed)\n\n# Get the wind direction.\nwind_direction = weather.get_wind_direction()\nprint(wind_direction)\n\n# Get the wind.\nwind = weather.get_wind()\nprint(wind)\n```\n\n## References\n\n- ### Gazetteer Files\n\n  - <https://www.census.gov/geographies/reference-files/time-series/geo/gazetteer-files.html>\n\n- ### API Documentation\n\n  - <https://www.weather.gov/documentation/services-web-api>\n\n- ### API Discussion\n\n  - <https://github.com/weather-gov/api/discussions>\n\n- ### Outage & Status Messages\n\n  - <https://www.nco.ncep.noaa.gov/status/messages/>\n\n## TODO\n\n- Get severe weather alerts.\n  - Active Alerts\n    - <https://api.weather.gov/alerts/active?point=38.9807,-76.9373>\n  - All Alerts\n    - <https://api.weather.gov/alerts?point=38.9807,-76.9373>\n',
    'author': 'Aaron Britton',
    'author_email': 'brittonleeaaron@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
